@@ -96,7 +96,7 @@ BEGIN {
     require Exporter;
 
     # set the version for version checking
-    our $VERSION   = 4.23;
+    our $VERSION   = 4.24;
     our @ISA       = qw(Exporter);
     our @EXPORT    = qw();
     our @EXPORT_OK = qw();
@@ -162,6 +162,29 @@ Currently, the only accepted values are either 1 for RGB or
 
 The default is 0 for BGR
 
+=item B< FOREGROUND >
+
+Sets the default foreground color for when 'attribute_reset'
+is called.  It is in the same format as "set_color"
+expects:
+
+ { # This is the default value
+   'red'   => 255,
+   'green' => 255,
+   'blue'  => 255
+ }
+=item B< BACKGROUND >
+
+Sets the default background color for when 'attribute_reset'
+is called.  It is in the same format as "set_b_color"
+expects:
+
+ { # This is the default value
+   'red'   => 0,
+   'green' => 0,
+   'blue'  => 0
+ }
+
 =item B< VXRES (Emulation Mode Only!  Otherwise ignored.)>
 
  Width of the emulation framebuffer.  Default is 640.
@@ -186,7 +209,7 @@ The default is 0 for BGR
 
 sub new {
     my $class = shift;
-    my @dummy;    # Just a temporary generic array for excess data returned from _get_info
+    my @dummy;    # Just a temporary generic array for excess data returned from _get_ioctl
     my $self = {
         'SCREEN'  => '', # The all mighty framebuffer
 
@@ -211,6 +234,16 @@ sub new {
         'PIE'         => 1,
         'POLY_ARC'    => 2,
         'FILE_MODE'   => FALSE,
+        'FOREGROUND'  => {
+            'red'   => 255,
+            'green' => 255,
+            'blue'  => 255
+        },
+        'BACKGROUND'  => {
+            'red'   => 0,
+            'green' => 0,
+            'blue'  => 0
+        },
 
         ## Set up the Framebuffer driver "constants" defaults
         # Commands
@@ -270,12 +303,12 @@ sub new {
             $self->{'yoffset'}, $self->{'bits_per_pixel'}, $self->{'grayscale'}, $self->{'bitfields'}, $self->{'nonstd'},
             $self->{'activate'}, $self->{'height'}, $self->{'width'}, $self->{'accel_flags'}, $self->{'pixclock'},
             $self->{'left_margin'}, $self->{'right_margin'}, $self->{'upper_margin'}, $self->{'lower_margin'},
-            $self->{'hsync_len'}, $self->{'vsync_len'}, $self->{'sync'}, $self->{'vmode'}, @dummy) = _get_info($self->{'FBIOGET_VSCREENINFO'}, $self->{'FBioget_vscreeninfo'}, $self->{'FB'});
+            $self->{'hsync_len'}, $self->{'vsync_len'}, $self->{'sync'}, $self->{'vmode'}, @dummy) = _get_ioctl($self->{'FBIOGET_VSCREENINFO'}, $self->{'FBioget_vscreeninfo'}, $self->{'FB'});
 
         # MAke the IOCTL call to get info on the physical screen
         ($self->{'id'}, $self->{'smem_start'}, $self->{'smem_len'}, $self->{'type'}, $self->{'type_aux'}, $self->{'visual'},
             $self->{'xpanstep'}, $self->{'ypanstep'}, $self->{'ywrapstep'}, $self->{'line_length'},
-            $self->{'mmio_start'}, $self->{'mmio_len'}, $self->{'accel'}, @dummy) = _get_info($self->{'FBIOGET_FSCREENINFO'}, $self->{'FBioget_fscreeninfo'}, $self->{'FB'});
+            $self->{'mmio_start'}, $self->{'mmio_len'}, $self->{'accel'}, @dummy) = _get_ioctl($self->{'FBIOGET_FSCREENINFO'}, $self->{'FBioget_fscreeninfo'}, $self->{'FB'});
 
         $self->{'VXRES'}          = $self->{'xres_virtual'};
         $self->{'VYRES'}          = $self->{'yres_virtual'};
@@ -372,13 +405,35 @@ Sets or returns the drawing mode, depending on how it is called.
 
 =over 1
 
- my $draw_mode = $fb->draw_mode();
+ my $draw_mode = $fb->draw_mode();     # Returns the current
+                                       # Drawing mode.
 
- $fb->draw_mode($fb->{'NORMAL_MODE'});
- $fb->draw_mode($fb->{'XOR_MODE'});
- $fb->draw_mode($fb->{'OR_MODE'});
- $fb->draw_mode($fb->{'AND_MODE'});
- $fb->draw_mode($fb->{'MASK_MODE'});
+ # Modes explained.  These settings are global
+
+                                       # When you draw it...
+
+ $fb->draw_mode($fb->{'NORMAL_MODE'}); # Replaces the screen pixel
+                                       # with the new pixel.
+
+ $fb->draw_mode($fb->{'XOR_MODE'});    # Does a bitwise XOR with
+                                       # the new pixel and screen
+                                       # pixel.
+
+ $fb->draw_mode($fb->{'OR_MODE'});     # Does a bitwise OR with
+                                       # the new pixel and screen
+                                       # pixel.
+
+ $fb->draw_mode($fb->{'AND_MODE'});    # Does a bitwise AND with
+                                       # the new pixel and screen
+                                       # pixel.
+
+ $fb->draw_mode($fb->{'MASK_MODE'});   # Draws the new pixel on
+                                       # screen areas not equal to
+                                       # the background color.
+
+ $fb->draw_mode($fb->{'UNMASK_MODE'}); # Draws the new pixel on
+                                       # screen areas only equal to
+                                       # the background color.
 
 =back
 =cut
@@ -477,6 +532,23 @@ sub mask_mode {
     $self->draw_mode($self->{'MASK_MODE'});
 }
 
+=head2 unmask_mode
+
+This is an alias to draw_mode($fb->{'UNMASK_MODE'})
+
+=over 1
+
+ $fb->unmask_mode();
+
+=back
+
+=cut
+
+sub unmask_mode {
+    my $self = shift;
+    $self->draw_mode($self->{'UNMASK_MODE'});
+}
+
 =head2 clear_screen
 
 Fills the entire screen with the background color
@@ -528,9 +600,9 @@ sub attribute_reset {
 
     $self->{'X'} = 0;
     $self->{'Y'} = 0;
-    $self->set_color({'red' => 255, 'green' => 255, 'blue' => 255});
+    $self->set_color($self->{'FOREGROUND'});
     $self->{'DRAW_MODE'} = $self->{'NORMAL_MODE'};
-    $self->set_b_color({'red' => 0, 'green' => 0, 'blue' => 0});
+    $self->set_b_color($self->{'BACKGROUND'});
     $self->clip_reset;
 } ## end sub attribute_reset
 
@@ -561,7 +633,13 @@ drawn with round pixels.  Square pixels are much faster.
 
 =over 1
 
-$fb->plot({'x' => 20,'y' => 30, 'pixel_size' => 3});
+$fb->plot(
+    {
+        'x'          => 20,
+        'y'          => 30,
+        'pixel_size' => 3
+    }
+);
 
 =back
 
@@ -587,13 +665,9 @@ sub plot {
 
         # Only plot if the pixel is within the clipping region
         unless (($x > $self->{'XX_CLIP'}) || ($y > $self->{'YY_CLIP'}) || ($x < $self->{'X_CLIP'}) || ($y < $self->{'Y_CLIP'})) {
-            # SV* smem_start, SV* smem_length, SV* bits_per_pixel, SV* bytes, SV* line_length,
-            # SV* xoffset, SV* yoffset, SV* width, SV* height, SV* draw_mode, SV* x, SV* y
-
-
             # The 'history' is a 'draw_arc' optimization and beautifier for xor mode.  It only draws pixels not in
             # the history buffer.
-            unless (exists($self->{'history'}) && defined($self->{'history'}->{"$y"}->{"$x"})) {
+            unless (exists($self->{'history'}) && defined($self->{'history'}->{$y}->{$x})) {
                 $index = ($self->{'BYTES_PER_LINE'} * ($y + $self->{'YOFFSET'})) + ($x * $self->{'BYTES'});
                 if ($index >= 0 && $index <= ($self->{'smem_len'} - $self->{'BYTES'})) {
                     eval {
@@ -632,7 +706,7 @@ sub plot {
                     };
                     $self->_fix_mapping() if ($@);
                 } ## end if ($index >= 0 && $index...)
-                $self->{'history'}->{"$y"}->{"$x"} = 1 if (exists($self->{'history'}));
+                $self->{'history'}->{$y}->{$x} = 1 if (exists($self->{'history'}));
             } ## end unless (exists($self->{'history'...}))
         }
     } ## end else [ if ($size > 1) ]
@@ -648,10 +722,10 @@ point xx,yy.  Clipping applies.
 =over 1
 
  $fb->line({
-    'x' => 50,
-    'y' => 60,
-    'xx' => 100,
-    'yy' => 332
+    'x'          => 50,
+    'y'          => 60,
+    'xx'         => 100,
+    'yy'         => 332
     'pixel_size' => 3
  });
 
@@ -663,9 +737,39 @@ sub line {
     my $self   = shift;
     my $params = shift;
 
-    $params->{'pixel_size'} = 1 unless(exists($params->{'pixel_size'}));
     $self->plot($params);
-    $self->drawto({'x' => $params->{'xx'},'y' => $params->{'yy'}, 'pixel_size' => $params->{'pixel_size'}});
+    $params->{'x'} = $params->{'xx'};
+    $params->{'y'} = $params->{'yy'};
+    $self->drawto($params);
+}
+
+=head2 angle_line
+
+Draws a line, in the global color, from point x,y
+at an angle of 'angle', of length 'radius'.  Clipping
+applies.
+
+=over 1
+
+ $fb->angle_line({
+    'x'          => 50,
+    'y'          => 60,
+    'radius'     => 50,
+    'angle'      => 30.3,
+    'pixel_size' => 3
+ });
+
+=back
+
+=cut
+
+sub angle_line {
+    my $self   = shift;
+    my $params = shift;
+
+    $params->{'xx'} = $params->{'radius'} * cos($params->{'angle'});
+    $params->{'yy'} = $params->{'radius'} * sin($params->{'angle'});
+    $self->line($params);
 }
 
 =head2 drawto
@@ -749,21 +853,25 @@ sub drawto {
                 }
             } ## end else [ if ($start_x > $x_end)]
         } else {
-            for (my $ty = ($y_end - ($size / 2)) ; $ty <= ($y_end + ($size / 2)) ; $ty++) {
+#            for (my $ty = ($y_end - ($size / 2)) ; $ty <= ($y_end + ($size / 2)) ; $ty++) {
                 if ($start_x > $x_end) {
                     if ($x_end < 0) {
-                        $self->blit_write({'x' => 0, 'y' => $ty, 'width' => $width + $x_end, 'height' => 1, 'image' => $self->{'COLOR'} x ($width + $x_end)}) if (($width + $x_end) > 0);           # Blitting a horizontal line is much faster!
+                        $self->blit_write({'x' => 0, 'y' => ($y_end - ($size / 2)), 'width' => $width + $x_end, 'height' => $size, 'image' => $self->{'COLOR'} x (($width + $x_end) * $size)}) if (($width + $x_end) > 0);           # Blitting a horizontal line is much faster!
+#                        $self->blit_write({'x' => 0, 'y' => $ty, 'width' => $width + $x_end, 'height' => 1, 'image' => $self->{'COLOR'} x ($width + $x_end)}) if (($width + $x_end) > 0);           # Blitting a horizontal line is much faster!
                     } else {
-                        $self->blit_write({'x' => $x_end, 'y' => $ty, 'width' => $width, 'height' => 1, 'image' => $self->{'COLOR'} x $width});                                                     # Blitting a horizontal line is much faster!
+                        $self->blit_write({'x' => $x_end, 'y' => ($y_end - ($size / 2)), 'width' => $width, 'height' => $size, 'image' => $self->{'COLOR'} x ($width * $size)});                                                     # Blitting a horizontal line is much faster!
+#                        $self->blit_write({'x' => $x_end, 'y' => $ty, 'width' => $width, 'height' => 1, 'image' => $self->{'COLOR'} x $width});                                                     # Blitting a horizontal line is much faster!
                     }
                 } else {
                     if ($start_x < 0) {
-                        $self->blit_write({'x' => 0, 'y' => $ty, 'width' => $width + $start_x, 'height' => 1, 'image' => $self->{'COLOR'} x ($width + $start_x)}) if (($width + $start_x) > 0);     # Blitting a horizontal line is much faster!
+                        $self->blit_write({'x' => 0, 'y' => ($y_end - ($size / 2)), 'width' => $width + $start_x, 'height' => $size, 'image' => $self->{'COLOR'} x (($width + $start_x) * $size)}) if (($width + $start_x) > 0);     # Blitting a horizontal line is much faster!
+#                        $self->blit_write({'x' => 0, 'y' => $ty, 'width' => $width + $start_x, 'height' => 1, 'image' => $self->{'COLOR'} x ($width + $start_x)}) if (($width + $start_x) > 0);     # Blitting a horizontal line is much faster!
                     } else {
-                        $self->blit_write({'x' => $start_x, 'y' => $ty, 'width' => $width, 'height' => 1, 'image' => $self->{'COLOR'} x $width});                                                   # Blitting a horizontal line is much faster!
+                        $self->blit_write({'x' => $start_x, 'y' => ($y_end - ($size / 2)), 'width' => $width, 'height' => $size, 'image' => $self->{'COLOR'} x ($width * $size)}); # Blitting a horizontal line is much faster!
+#                        $self->blit_write({'x' => $start_x, 'y' => $ty, 'width' => $width, 'height' => 1, 'image' => $self->{'COLOR'} x $width});                                                   # Blitting a horizontal line is much faster!
                     }
                 } ## end else [ if ($start_x > $x_end)]
-            } ## end for (my $ty = ($y_end -...))
+#            } ## end for (my $ty = ($y_end -...))
         } ## end else [ if ($size == 1) ]
         $self->plot({'x' => $x_end, 'y' => $y_end, 'pixel_size' => $size});
     } elsif ($width > $height) {    # Wider than it is high
@@ -1163,18 +1271,52 @@ sub ellipse {
     my $StoppingY    = 0;
 
     $self->{'history'} = {} unless ($filled || $size > 1);
+    my ($red,$green,$blue,$rc,$gc,$bc);
+    my $gradient = FALSE;
+    my $saved = $self->{'COLOR'};
+    my @grad;
+    if (exists($params->{'gradient'})) {
+        my $ydiameter = $YRadius * 2;
+        $red   = $params->{'gradient'}->{'start'}->{'red'};
+        $green = $params->{'gradient'}->{'start'}->{'green'};
+        $blue  = $params->{'gradient'}->{'start'}->{'blue'};
+        $rc = ($red   <=> $params->{'gradient'}->{'end'}->{'red'})   ? ($params->{'gradient'}->{'end'}->{'red'}   - $red)   / $ydiameter : 0;
+        $gc = ($green <=> $params->{'gradient'}->{'end'}->{'green'}) ? ($params->{'gradient'}->{'end'}->{'green'} - $green) / $ydiameter : 0;
+        $bc = ($blue  <=> $params->{'gradient'}->{'end'}->{'blue'})  ? ($params->{'gradient'}->{'end'}->{'blue'}  - $blue)  / $ydiameter : 0;
+        $gradient = TRUE;
+        # Make an array predefining the gradient on the y axis, then use it to set the color below
+        push(@grad,[$red,$green,$blue]);
+        foreach $yg (1..$ydiameter) {
+            $red   = abs($red   + $rc) if ($rc);
+            $green = abs($green + $gc) if ($gc);
+            $blue  = abs($blue  + $bc) if ($bc);
+            push(@grad,[$red,$green,$blue]);
+        }
+        $gradient = TRUE;
+    }
 
     while ($StoppingX >= $StoppingY) {
         my $cxx  = int($cx + $x);
         my $cx_x = int($cx - $x);
         my $cyy  = int($cy + $y);
         my $cy_y = int($cy - $y);
+        my $rpy  = $YRadius + $y;
+        my $rmy  = $YRadius - $y;
+#        print "$rpy,$rmy,",scalar(@grad),"\n";
         if ($filled) {
             if ($cyy <=> $old_cyy) {
+                if ($gradient) {
+                    my ($r,$g,$b) = @{$grad[$rpy]};
+                    $self->set_color({'red' => $r,'green' => $g, 'blue' => $b});
+                }
                 $self->line({'x' => $cxx, 'y' => $cyy,'xx' => $cx_x, 'yy' => $cyy});
                 $old_cyy = $cyy;
             }
             if (($cy_y <=> $old_cy_y) && ($cyy <=> $cy_y)) {
+                if ($gradient) {
+                    my ($r,$g,$b) = @{$grad[$rmy]};
+                    $self->set_color({'red' => $r,'green' => $g, 'blue' => $b});
+                }
                 $self->line({'x' => $cx_x, 'y' => $cy_y,'xx' => $cxx, 'yy' => $cy_y});
                 $old_cy_y = $cy_y;
             }
@@ -1208,12 +1350,22 @@ sub ellipse {
         my $cx_x = int($cx - $x);
         my $cyy  = int($cy + $y);
         my $cy_y = int($cy - $y);
+        my $rpy  = $YRadius + $y;
+        my $rmy  = $YRadius - $y;
         if ($filled) {
             if ($cyy <=> $old_cyy) {
+                if ($gradient) {
+                    my ($r,$g,$b) = @{$grad[$rpy]};
+                    $self->set_color({'red' => $r,'green' => $g, 'blue' => $b});
+                }
                 $self->line({'x' => $cxx, 'y' => $cyy, 'xx' => $cx_x, 'yy' => $cyy});
                 $old_cyy = $cyy;
             }
             if (($cy_y <=> $old_cy_y) && ($cyy <=> $cy_y)) {
+                if ($gradient) {
+                    my ($r,$g,$b) = @{$grad[$rmy]};
+                    $self->set_color({'red' => $r,'green' => $g, 'blue' => $b});
+                }
                 $self->line({'x' => $cx_x, 'y' => $cy_y, 'xx' => $cxx, 'yy' => $cy_y});
                 $old_cy_y = $cy_y;
             }
@@ -1235,6 +1387,7 @@ sub ellipse {
         } ## end if ((($EllipseError * ...)))
     } ## end while ($StoppingX <= $StoppingY)
     delete($self->{'history'}) if (exists($self->{'history'}));
+    $self->{'COLOR'} = $saved;
 } ## end sub ellipse
 
 =head2 circle
@@ -1258,13 +1411,11 @@ sub circle {
     my $self   = shift;
     my $params = shift;
 
-    my $x      = $params->{'x'};
-    my $y      = $params->{'y'};
-    my $r      = $params->{'radius'};
-    my $filled = $params->{'filled'}     || 0;
-    my $size   = $params->{'pixel_size'} || 1;
+    $params->{'xradius'} = $r;
+    $params->{'yradius'} = $r;
+    $params->{'factor'}  = 1;
     
-    $self->ellipse({'x' => $x, 'y' => $y, 'xradius' => $r, 'yradius' => $r, 'filled' => $filled, 'factor' => 1, 'pixel_size' => $size});
+    $self->ellipse($params);
 } ## end sub circle
 
 =head2 polygon
@@ -1310,23 +1461,49 @@ sub polygon {
     }
     $self->drawto({'x' => $xx, 'y' => $yy, 'pixel_size' => $size});
     $self->plot({'x' => $xx, 'y' => $yy, 'pixel_size' => $size}) if ($self->{'DRAW_MODE'} == 1);
-    $self->_fill_history();
+    $self->_fill_history($params);
 } ## end sub polygon
 
 sub _fill_history {
     my $self = shift;
+    my $p    = shift;
+    my $gradient = FALSE;
     if (exists($self->{'history'})) {
+
         my %c = %{$self->{'history'}};
         delete($self->{'history'});
-        foreach my $Y (sort(keys %c)) {
-            my @k = (sort(keys %{$c{$Y}}));
+        my @ys = sort {$a <=> $b} (keys %c);
+
+        my ($red,$green,$blue,$rc,$gc,$bc);
+        if (exists($p->{'gradient'})) {
+            $red   = $p->{'gradient'}->{'start'}->{'red'};
+            $green = $p->{'gradient'}->{'start'}->{'green'};
+            $blue  = $p->{'gradient'}->{'start'}->{'blue'};
+            $rc = ($red   <=> $p->{'gradient'}->{'end'}->{'red'}   && scalar(@ys)) ? ($p->{'gradient'}->{'end'}->{'red'}   - $red)   / scalar(@ys) : 0;
+            $gc = ($green <=> $p->{'gradient'}->{'end'}->{'green'} && scalar(@ys)) ? ($p->{'gradient'}->{'end'}->{'green'} - $green) / scalar(@ys) : 0;
+            $bc = ($blue  <=> $p->{'gradient'}->{'end'}->{'blue'}  && scalar(@ys)) ? ($p->{'gradient'}->{'end'}->{'blue'}  - $blue)  / scalar(@ys) : 0;
+            $gradient = TRUE;
+        }
+        my $saved = $self->{'COLOR'};
+        foreach my $Y (@ys) {
+            my @k = (sort {$a <=> $b} (keys %{$c{$Y}}));
+            if ($gradient) {
+                $self->set_color({'red' => $red, 'green' => $green, 'blue' => $blue});
+                $red   = abs($red   + $rc) if ($rc);
+                $green = abs($green + $gc) if ($gc);
+                $blue  = abs($blue  + $bc) if ($bc);
+            }
             while(scalar(@k) > 1) {
                 my $x1 = shift(@k);
                 my $x2 = shift(@k);
+                while (scalar(@k) && ($x2 - $x1) <= 1) {
+                    ($x1,$x2) = ($x2,shift(@k));
+                }
                 $self->line({'x' => $x1, 'y' => $Y, 'xx' => $x2, 'yy' => $Y});
             }
             $self->drawto({'x' => shift(@k),'y' => $Y}) if (scalar(@k));
         }
+        $self->{'COLOR'} = $saved;
     }
 }
 
@@ -1374,19 +1551,23 @@ sub box {
         ($y, $yy) = ($yy, $y);
     }
     if ($filled) {
-        $w = abs($xx - $x);
-        $h = abs($yy - $y);
-        if (($self->{'accel_flags'} & $self->{'FBINFO_HWACCEL_FILLRECT'}) && $self->{'DRAW_MODE'} <= 1) {
-            _set_info(
-                $self->{'FBINFO_HWACCEL_FILLRECT'},
-                $self->{'FBinfo_hwaccel_fillrect'},
-                $self->{'FB'},
-                $x,$y,$w,$h,
-                $self->{'COLOR'},
-                $self->{'DRAW_MODE'}
-            );
+        if (exists($params->{'gradient'})) {
+            $self->polygon({'pixel_size' => $size,'filled' => 1,'gradient' => $params->{'gradient'}, 'coordinates' => [$x, $y, $xx, $y, $xx, $yy, $x, $yy]});
         } else {
-            $self->blit_write({'x' => $x, 'y' => $y, 'width' => $w, 'height' => $h, 'image' => $self->{'COLOR'} x ($w * $h)});
+            $w = abs($xx - $x);
+            $h = abs($yy - $y);
+            if (($self->{'accel_flags'} & $self->{'FBINFO_HWACCEL_FILLRECT'}) && $self->{'DRAW_MODE'} <= 1) {
+                _set_ioctl(
+                    $self->{'FBINFO_HWACCEL_FILLRECT'},
+                    $self->{'FBinfo_hwaccel_fillrect'},
+                    $self->{'FB'},
+                    $x,$y,$w,$h,
+                    $self->{'COLOR'},
+                    $self->{'DRAW_MODE'}
+                );
+            } else {
+                $self->blit_write({'x' => $x, 'y' => $y, 'width' => $w, 'height' => $h, 'image' => $self->{'COLOR'} x ($w * $h)});
+            }
         }
     } else {
         $self->polygon({'pixel_size' => $size, 'coordinates' => [$x, $y, $xx, $y, $xx, $yy, $x, $yy]});
@@ -1424,12 +1605,11 @@ sub rbox {
     my $y      = $params->{'y'};
     my $w      = $params->{'width'};
     my $h      = $params->{'height'};
-    my $filled = $params->{'filled'} || 0;
-    my $size   = $params->{'pixel_size'};
-    $size = 1 if ($filled);
     my $xx = $x + $w;
     my $yy = $y + $h;
-    $self->box({'x' => $x, 'y' => $y, 'xx' => $xx, 'yy' => $yy, 'filled' => $filled, 'pixel_size' => $size});
+    $params->{'xx'} = $xx;
+    $params->{'yy'} = $yy;
+    $self->box($params);
 } ## end sub rbox
 
 =head2 set_color
@@ -1733,7 +1913,7 @@ sub blit_copy {
     my $yy = int($params->{'y_dest'});
 
     if (($self->{'accel_flags'} & $self->{'FBINFO_HWACCEL_COPYAREA'}) && $self->{'DRAW_MODE'} < 1) {
-        _set_info(
+        _set_ioctl(
             $self->{'FBINFO_HWACCEL_COPYAREA'},
             $self->{'FBinfo_hwaccel_copyarea'},
             $self->{'FB'},
@@ -2109,7 +2289,7 @@ Bounding box mode gets the actual values needed to display the text.
      'color'        => 'FFFF00', # Yellow
      'text'         => 'Hello World!',
      'font_path'    => '/usr/share/fonts/truetype',
-     'face'         => 'Arial',
+     'face'         => 'Arial.ttf',
      'bounding_box' => 1,
      'center'       => 0
  });
@@ -2130,7 +2310,7 @@ Here's a shortcut:
          'color'        => 'FFFF00', # Yellow
          'text'         => 'Hello World!',
          'font_path'    => '/usr/share/fonts/truetype',
-         'face'         => 'Arial',
+         'face'         => 'Arial.ttf',
          'bounding_box' => 1,
          'center'       => 0
      })
@@ -2172,7 +2352,7 @@ sub ttf_print {
             'file'  => "$font_path/$face",
             'color' => $P_color,
             'size'  => $TTF_h
-        );
+        ); # || print STDERR "ERROR, can't initialize Imager font engine\n";
 
         ($neg_width, $global_descent, $pos_width, $global_ascent, $descent, $ascent, $advance_width, $right_bearing) = $font->bounding_box('string' => $text, 'canon' => 1, 'size' => $TTF_h);
         if ($box_mode) {
@@ -2218,6 +2398,7 @@ sub ttf_print {
         );
         $self->blit_write({'x' => $TTF_x, 'y' => (($TTF_y - $TTF_h) + 1), 'width' => $TTF_w, 'height' => (($TTF_h + $global_ascent) - $global_descent), 'image' => $data});
     };
+#    print STDERR "ERROR $@\n" if ($@);
     return (
         {
             'color'        => $params->{'color'},
@@ -2380,7 +2561,7 @@ sub RGB_to_16 {
     # Converts a 24 bit pixel value to a 16 bit pixel value.                     #
     # -------------------------------------------------------------------------- #
     # This is not a fancy table based color conversion.  This merely uses math,  #
-    # and thus the quality is lacking on the output.                             #
+    # and thus the quality is lacking on the output.  RGB888 -> RGB565           #
     ##############################################################################
 
     my $self   = shift;
@@ -2392,11 +2573,16 @@ sub RGB_to_16 {
     while ($big_data ne '') {
         my $pixel_data = substr($big_data, 0, 3);
         $big_data = substr($big_data, 3) . chr(255);
-        my ($b, $g, $r, $a) = unpack('I', $pixel_data);
-        $r = int($r / 8);
-        $g = int($g / 8);
-        $b = int($b / 8);
-        my $color = ($r << 11) + ($g << 6) + $b;
+        my ($b, $g, $r) = unpack('L', $pixel_data);
+        $r = $r >> 3; # 5 bits for red
+        $g = $g >> 2; # 6 bits for green
+        $b = $b >> 3; # 5 bits for blue
+        my $color;
+        if ($self->{'COLOR_MODE'} == BGR) {
+            $color = ($b << 11) | ($g << 5) | $r;
+        } else {
+            $color = ($r << 11) | ($g << 5) | $b;
+        }
         $n_data .= pack('S', $color);
     } ## end while ($big_data ne '')
     return ({'color' => $n_data});
@@ -2416,7 +2602,7 @@ sub RGBA_to_16 {
     # -------------------------------------------------------------------------- #
     # This is not a fancy table based color conversion.  This merely uses math,  #
     # and thus the quality is lacking on the output.  This discards the alpha    #
-    # channel.                                                                   #
+    # channel.  RGB888A -> RGB565                                                #
     ##############################################################################
     my $self   = shift;
     my $params = shift;
@@ -2428,10 +2614,15 @@ sub RGBA_to_16 {
         my $pixel_data = substr($big_data, 0, 4);
         $big_data = substr($big_data, 4);
         my ($b, $g, $r, $a) = unpack('L', $pixel_data);
-        $r = int($r / 8);
-        $g = int($g / 8);
-        $b = int($b / 8);
-        my $color = ($r << 11) + ($g << 6) + $b;
+        $r = $r >> 3;
+        $g = $g >> 2;
+        $b = $b >> 3;
+        my $color;
+        if ($self->{'COLOR_MODE'} == BGR) {
+            $color = ($b << 11) | ($g << 5) | $r;
+        } else {
+            $color = ($r << 11) | ($g << 5) | $b;
+        }
         $n_data .= pack('S', $color);
     } ## end while ($big_data ne '')
     return ({'color' => $n_data});
@@ -2467,7 +2658,7 @@ sub _fix_mapping {
 }
 ## Just standard flat subroutines
 
-sub _get_info {
+sub _get_ioctl {
     ##########################################################
     ##                    GET IOCTL INFO                    ##
     ##########################################################
@@ -2481,9 +2672,9 @@ sub _get_info {
     ioctl($fb, $command, $data);
     @array = unpack($format, $data);
     return (@array);
-} ## end sub _get_info
+} ## end sub _get_ioctl
 
-sub _set_info {
+sub _set_ioctl {
     ##########################################################
     ##                    SET IOCTL INFO                    ##
     ##########################################################
@@ -2496,7 +2687,7 @@ sub _set_info {
 
     my $data    = pack($format, @array);
     ioctl($fb, $command, $data);
-} ## end sub _set_info
+} ## end sub _set_ioctl
 
 1;
 
@@ -2510,9 +2701,14 @@ The module can NOT have separate threads calling the same object.
 You WILL crash. However, you can instantiate an object for each
 thread to use, and it will work just fine!
 
-See the "examples" directory for "frtest.pl" as an example of a
-threading script that uses this module.  Just add the number of
-threads you want it to use to the command line when you run it.
+See the "examples" directory for "threadstest.pl" as an example
+of a threading script that uses this module.  Just add the number
+of threads you want it to use to the command line when you run it.
+
+head2 FORKS
+
+I have never tested with forks.  Do at your own risk, but follow
+the same rules as in threads, and it may work.
 
 =head2 BLITTING
 
@@ -2520,6 +2716,11 @@ Use blit_read and blit_write to save portions of the screen
 instead of redrawing everything.  It will speed up response
 tremendously.
 
+=head3 SPRITES
+
+Someone asked me about sprites.  Well, that's what blitting is
+for.  You'll have to do your own collision detection.
+
 =head2 HORIZONTAL "MAGIC"
 
 Horizontal lines and filled boxes draw very fast.  Learn to
@@ -2537,6 +2738,25 @@ When drawing a filled pie arc, try larger pixel sizes, in
 addition to tweaking the granularity.  Larger pixel sizes will
 fill in gaps as a result of higher granularities.
 
+=head2 MICROSOFT WINDOWS
+
+It doesn't work natively, (other than in emulation mode) and
+never will.  However...
+
+You can run Linux inside VirtualBox and it works fine.  Put it
+if full screen mode, and voila, it's "running in Windows" in an
+indirect way.  Make sure you install the VirtualBox extensions,
+as it has the correct video driver for framebuffer access.
+It's as close as you'll ever get to get it running in MS Windows.
+
+This isn't a design choice nor preference.  It's simply because
+of the fact Windows does not allow file mapping of the display,
+nor variable memory mapping of the display.  Both techniques
+this module uses to achieve its magic.  DirectX is more like
+OpenGL in how it works, and thus defeats the purpose of this
+module.  You're better off with SDL instead, if you want to draw
+in MS Windows.
+
 =head1 AUTHOR
 
 Richard Kelsch <rich@rk-internet.com>
@@ -2550,7 +2770,7 @@ modify it under the same terms as Perl itself.
 
 =head1 VERSION
 
-Version 4.23 (July 21, 2015)
+Version 4.24 (July 23, 2015)
 
 =head1 THANKS
 
